"""AsyncDB Meta information."""

__title__ = 'python-datamodel'
__description__ = ('simple library based on python +3.7 to use Dataclass-syntax'
                   'for interacting with Data')
__version__ = '0.0.18'
__author__ = 'Jesus Lara'
__author_email__ = 'jesuslarag@gmail.com'
__license__ = 'BSD'
